******************************************** Examples *************************************;

/* Setup sample data set */
libname fram 'C:\Users\gagnon\Dropbox\Theory\Reusable Code';
proc contents data=fram.fram;
run;
proc univariate data=fram.fram;
var sysbp;
run;
proc format; value $outf cvd='CVD' death='Death';
             value $parmf 'DIABETES'='Diabetes' 'BMIGRP'='BMI';
             value sexf 0='Female' 1='Male';
             value predf 1='< 24' 2='24-28' 3='> 28';
             value agef 0='< 65' 1='65+';
             run;
data one; set fram.fram (keep=randid age sex bmi diabetes hyperten death cvd time timecvd timedth period sysbp);
where period eq 3;
agecat = age >= 65;
if agecat eq 0 then agegrp = 0;
  else agegrp = 1;
sex = sex-1;
length hyp $ 50;
if . lt sysbp lt 130 then hyp = 'Not Even Close to Hypertensive';
else if 130 le sysbp lt 150 then hyp='Probably Borderline Hypertensive';
else hyp='Hypertensive';
if 24 le bmi le 28 then BMIgrp = 2;
else if bmi lt 24 then BMIgrp = 1;
  else if bmi gt 28 then BMIgrp = 3;
timecvd = timecvd-time;
timedth = timedth-time;
/* Create offsets as the log(time) */
if timecvd gt 0 then offcvd = log(timecvd);
if timedth gt 0 then offdth = log(timedth);
keep CVD Death BMIgrp Sex Diabetes agegrp hyp sysbp RandID timedth timecvd offdth offcvd age;
format sex sexf.  agegrp agef.;
label agegrp='Age ';
label sex = 'SEX';
label bmigrp='BMI';
run;
proc means data=one;
run;
proc freq data=one;
tables hyp;
run;
/* Turn macro debugging on: 
 delete MPRINT file
   Full debugging with MPRINT output 

data _null_;
    fname="tempfile";
    rc=filename(fname, "C:\Users\gagnon\Dropbox\Theory\Reusable Code\mprint\mprint1.sas");
    if rc = 0 and fexist(fname) then
       rc=fdelete(fname);
    rc=filename(fname);
run;

/* Full debugging with MPRINT output
filename mprint 'C:\Users\gagnon\Dropbox\Theory\Reusable Code\mprint\mprint1.sas';
options mprint mfile symbolgen spool mlogic VALIDVARNAME=V7 ls=80 ps=1000;run;
*/

/*Turn off macro debugging */
options noSYMBOLGEN noMPRINT noMLOGIC SPOOL VALIDVARNAME=V7 ls=80 ps=1000;

/********************************** Example 1: A simple example ************************************/

/* %fdataphc1 macro with DEATH and CVD as outcomes, 
   TIMEDTH and TIMECVD as survival times, AGEGRP and SEX as BY variables, 
   BMIgrp as the sole, categorical main predictor and DIABETES as a covariate.
*/

/* Run the regression models and prepare report and forest plot data sets */
 ods listing;
%fdataphc1(dsn= one,outcomes= Death CVD,surv0=,surv1=timedth timecvd,weight=,
         byvars= agegrp sex,mainpred= BMIgrp,covars= Diabetes hyp,
         clvars= bmigrp (ref='2') diabetes (ref='0') hyp,outdata= repphc1);
run;
ods listing;
proc contents data=repphc1;
run;
proc print data=repphc1;
var outcome agegrp sex betaname2 meantime EV_subs PECI probchisq scolor multout;
run;
/* Print the reports */
%repmakec(indata=repphc1,byvars= agegrp sex,
          dest=ods excel file="C:\Users\gagnon\Dropbox\Theory\ODS graphics\tableout1.xlsx");
run;
/* Make the forest plots as PNG pixel image files */
 ods listing  gpath="C:\Users\gagnon\Documents\figure output" ;
run;
 ods graphics on / width=8in height=5in IMAGEFMT=png;
%mforest(plotdata=repphc1,outcount= 2,byvars= agegrp sex);
run;

/* Forest plots to Excel */
ods graphics on / width=8in height=10in IMAGEFMT=png;
ods excel file="C:\Users\gagnon\Dropbox\Theory\ODS graphics\figures1.xlsx";
%mforest(plotdata=repphc1,outcount=2,byvars= agegrp sex);
run;
ods excel close;

/* Make the forest plots as SVG vector graphics files */
 ods listing;
 ods graphics on / width=8in height=10in IMAGEFMT=svg;
%mforest(plotdata=repphc1,outcount= 2,byvars= agegrp sex);
run;
proc print data=repphc1;
run;

/**************************** Example 2: combining multiple models *****************************/

/* %fdataphc1 macro with DEATH and CVD as outcomes, TIMEDTH and TIMECVD as 
            survival times, no BY variables, BMIgrp as the sole, 
            categorical main predictor and Diabetes, age and sex as a covariate.
        ****Also, do crude, age and multivariate adjusted models.
*/

/* Run the regression models and prepare report and forest plot data sets */
 ods listing;
 /* UNadjusted */
%fdataphc1(dsn= one,outcomes= Death CVD,surv0=,surv1=timedth timecvd,weight=,
         byvars=,mainpred= BMIgrp,covars=,
         clvars= bmigrp (ref='2'),outdata= Ex3ua);
run;

/* Age adjusted */
%fdataphc1(dsn= one,outcomes= Death CVD,surv0=,surv1=timedth timecvd,weight=,
         byvars=,mainpred= BMIgrp,covars= age,
         clvars= bmigrp (ref='2') ,outdata= Ex3aa);
run;

/* MV Adjusted */
%fdataphc1(dsn= one,outcomes= Death CVD,surv0=,surv1=timedth timecvd,weight=,
         byvars=,mainpred= BMIgrp,covars= Diabetes age sex hyp,
         clvars= bmigrp (ref='2') diabetes (ref='0') hyp,outdata= Ex3mv);
run;

/* Make any modifications to Report data sets */
data recalc1; set Ex3ua;
x1= (meantime*nsubs/30); *convert mean person-time and number of subjects to total person-months;
x2 = put((dsubs/x1)*1000,4.3); *Events/1000 person months;
if x1=. then dsubs=.;
run; 
data recalc2; set Ex3aa;
x1= (meantime*nsubs/30); *convert mean person-time and number of subjects to total person-months;
x2 = put((dsubs/x1)*1000,4.3); *Events/1000 person months;
if x1=. then dsubs=.;
run;
data recalc3; set Ex3mv;
x1= (meantime*nsubs/30); *convert mean person-time and number of subjects to total person-months;
x2 = put((dsubs/x1)*1000,4.3); *Events/1000 person months;
if x1=. then dsubs=.;
run;
proc contents data=recalc3;
run;

/* Read template file */
proc import 
  datafile="C:\Users\gagnon\Dropbox\Theory\Reports\multifile.xlsx" 
  dbms=xlsx 
  out=work.templateM 
  replace;
run;
proc print data=templateM;
run;
/* Read shuffle file */
proc import 
  datafile="C:\Users\gagnon\Dropbox\Theory\Reports\Melissa 3.xlsx" 
  dbms=xlsx 
  out=work.shuffleM
  replace;
run;
proc print data=shuffleM;
run;
/* delete MPRINT file */
data _null_;
    fname="tempfile";
    rc=filename(fname, "C:\Users\gagnon\Dropbox\Theory\Reusable Code\mprint\mprint1.sas");
    if rc = 0 and fexist(fname) then
       rc=fdelete(fname);
    rc=filename(fname);
run;

/* Full debugging with MPRINT output */
filename mprint 'C:\Users\gagnon\Dropbox\Theory\Reusable Code\mprint\mprint1.sas';
options mprint mfile symbolgen spool mlogic VALIDVARNAME=V7 ls=80 ps=1000;run;

%fmergitr(template=templateM, shuffle=shuffleM,outrep=shuffledM,multout=1);
run;


/* Reorder the output */
proc sort data=shuffledM; by outcome norder classval0;
run;

proc contents data=shuffledM;
run;
proc print data=shuffledM;
var outcome stratum betaname2 peci norder classval0;
run;
/* Restore correct shading 
NOTE: available sorting variables to use are STORDER for strata, OORDER for outcomes
and BORDER for betaname2. */
data shuffledMa; set shuffledM; by outcome norder classval0;
retain korder;
storder = norder;
if _n_ eq 1 then korder = 0;
if first.norder then korder = korder+1;
scolor = mod(korder,2) in (1);
format probchisq pvalue6.4;
run;
ods listing;
proc print data=shuffledMa;
var outcome stratum betaname2 peci storder  classval0 sorder;
run;
%repmakec(indata=shuffledMa,byvars= ,
          dest=ods excel file="C:\Users\gagnon\Dropbox\Theory\ODS graphics\shuffledMa.xlsx");
run;

 ods listing  gpath="C:\Users\gagnon\Documents\figure output" ;
run;
 ods graphics on / width=8in height=5in IMAGEFMT=png;
%mforest(plotdata=shuffledMa,outcount=2,byvars=);
run;

/************************ Variations on the Reports, with Model as last level ************************/
/* Read new shuffle file */
proc import 
  datafile="C:\Users\gagnon\Dropbox\Theory\Reports\Melissa 3a.xlsx" 
  dbms=xlsx 
  out=work.shuffleMz
  replace;
run;
proc print data=shuffleMz;
run;
/* delete MPRINT file */
data _null_;
    fname="tempfile";
    rc=filename(fname, "C:\Users\gagnon\Dropbox\Theory\Reusable Code\mprint\mprint1.sas");
    if rc = 0 and fexist(fname) then
       rc=fdelete(fname);
    rc=filename(fname);
run;

/* Full debugging with MPRINT output */
filename mprint 'C:\Users\gagnon\Dropbox\Theory\Reusable Code\mprint\mprint1.sas';
options mprint mfile symbolgen spool mlogic VALIDVARNAME=V7 ls=80 ps=1000;run;

%fmergitr(template=templateM, shuffle=shuffleMz,outrep=shuffledMz,multout=1);
run;

/* Make the Reports */

%repmakec(indata=shuffledMz,byvars= ,
          dest=ods excel file="C:\Users\gagnon\Dropbox\Theory\ODS graphics\Mversion2.xlsx");
run;

/* Set the shading for the forest plots */
proc sort data=shuffledMz; by outcome  classval0 norder;
run;
proc print data=shuffledMz;
*var outcome stratum betaname2;
run;
proc contents data=shuffledMz;
run;
data shuffledMza; set shuffledMz;by outcome  classval0 norder;
retain korder;
if _n_ eq 1 then korder = 0;
if (first.classval0) and (classval0 ne 2) then korder = korder+1;
if mod(korder,2) in (1) then fshade = 1;
sorder = classval0;
border = norder;
if stratum eq '< 24' then stratum='BMI < 24';
if stratum eq '24-28' then stratum='BMI 24-28';
if stratum eq '> 28' then stratum='BMI > 28';
run;
proc print data=shuffledMza;
var outcome stratum sorder classval0 korder betaname2 norder border fshade;
run;
 ods listing  gpath="C:\Users\gagnon\Documents\figure output" ;
run;
 ods graphics on / width=8in height=5in IMAGEFMT=png;
%mforest(plotdata=shuffledMza,outcount=2,byvars=);
run;


********************** Example 3: PHREG with STRATA variables ************************************;
%fdataphc2(dsn= one,outcomes= Death CVD,surv0=,surv1=timedth timecvd,weight=,
         stratvars= agegrp sex,mainpred= BMIgrp,covars= Diabetes hyp,
         clvars= bmigrp (ref='2') diabetes (ref='0') hyp,
         outdata=fda2data);
run;
proc print data=fda2data;
run;
%repmakec(indata=fda2data,byvars= ,
          dest=ods excel file="C:\Users\gagnon\Dropbox\Theory\ODS graphics\fdac2.xlsx");
run;
ods listing;
 ods graphics on / width=8in height=10in IMAGEFMT=png;
/* Create the Plot */
%mforest(plotdata=fda2data,outcount=2,byvars=);
run;
/******************* Example 4:GENMOD Categorical Version ***************************/

/* A:  BY version, Binary distribution, logit link */

%fdatagmc1(dsn= one,outcomes= Death CVD,dist=bin,link=logit,offset=,weight=,expyn=YES,
         byvars= agegrp sex,mainpred= bmigrp,covars= Diabetes hyp,
         clvars= bmigrp (ref='2') diabetes (ref='0') hyp,
         outdata=fdagm1adata);
run;

%repmakec(indata=fdagm1adata,byvars= agegrp sex,
          dest=ods excel file="C:\Users\gagnon\Dropbox\Theory\ODS graphics\fdagm1a.xlsx");
run;


/* Create the Plot */
ods listing;
 ods graphics on / width=8in height=10in IMAGEFMT=png;
%mforest(fdagm1adata,outcount=2,byvars=agegrp sex);
run;

/* B:  BY version, Poisson distribution, log link, with offset */

%fdatagmc1(dsn= one,outcomes= Death CVD,dist=poisson,link=log,offset=offdth offcvd,weight=,expyn=YES,bout=YES,
         byvars= agegrp sex,mainpred= bmigrp,covars= Diabetes hyp,
         clvars= bmigrp (ref='2') diabetes (ref='0') hyp,
         outdata=fdagm1bdata);

proc contents data=fdagm1bdata out=conoutx;run;
proc contents data=ptimes out=conoutx;run;

%repmakec(indata=fdagm1bdata,byvars= agegrp sex,
          dest=ods excel file="C:\Users\gagnon\Dropbox\Theory\ODS graphics\fdagm1b.xlsx");
run;
/* Create the Plot */
ods listing;
 ods graphics on / width=8in height=10in IMAGEFMT=png;
%mforest(plotdata=fdagm1bdata,outcount=2,byvars= agegrp sex);
run;

proc print data=fplotit1;
run;

/* C:  STRATUM version, Binary distribution, logit link */

ods listing;
%fdatagmc2(dsn= one,outcomes= Death CVD,dist=bin,link=logit,offset=,weight=,expyn=YES,
         stratvars= agegrp sex,mainpred= bmigrp,covars= Diabetes hyp,
         clvars= bmigrp (ref='2') diabetes (ref='0') hyp,
         outdata=fdagm2cdata,outplot=fdagm2cplot);
run;
%repmakec(indata=fdagm2cdata,byvars= agegrp sex,
          dest=ods excel file="C:\Users\gagnon\Dropbox\Theory\ODS graphics\fdagm2c.xlsx");
run;
/* Create the Plot */
%mforest(fdagm2cplot);
run;


/* D:  STRATUM version, Poisson distribution, log link, with offset */

%fdatagmc2(dsn= one,outcomes= Death CVD,dist=poisson,link=log,offset=offdth offcvd,weight=,expyn=YES,
         stratvars= agegrp sex,mainpred= bmigrp,covars= Diabetes hyp,
         clvars= bmigrp (ref='2') diabetes (ref='0') hyp,
         outdata=fdagm2ddata,outplot=fdagm2dplot);
run;

%repmakec(indata=fdagm2ddata,byvars= agegrp sex,
          dest=ods excel file="C:\Users\gagnon\Dropbox\Theory\ODS graphics\fdagm2d.xlsx");
run;
/* Create the Plot */
%mforest(fdagm2dplot);
run;


